# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-04-19 11:56
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('message_sender', '0007_add_to_addr_index'),
        ('message_sender', '0009_auto_20170323_1453'),
    ]

    operations = [
    ]
