# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-04-19 16:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('message_sender', '0012_channel_insert'),
    ]

    operations = [
        migrations.AddField(
            model_name='outbound',
            name='channel',
            field=models.ForeignKey(null=True, on_delete=models.deletion.CASCADE, to='message_sender.Channel'),
        ),
    ]
