# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2018-01-31 10:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('message_sender', '0015_identitylookup'),
    ]

    operations = [
        migrations.AddField(
            model_name='outbound',
            name='resend',
            field=models.NullBooleanField(default=None, help_text=b'True if this is a resend requested by the user.'),
        ),
    ]
